# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc, 2017 Nokia
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.




from .fetchers import NUMetadatasFetcher


from .fetchers import NUAlarmsFetcher


from .fetchers import NUGlobalMetadatasFetcher


from .fetchers import NUContainerInterfacesFetcher


from .fetchers import NUContainerResyncsFetcher


from .fetchers import NUVRSsFetcher


from .fetchers import NUEventLogsFetcher

from bambou import NURESTObject


class NUContainer(NURESTObject):
    """ Represents a Container in the VSD

        Notes:
            API that can retrieve the containers associated with a domain, zone or subnet for mediation created containers for REST created  containers you need to set the additional proxy user header in http request : X-Nuage-ProxyUservalue of the header has to be either :1) enterpriseName@UserName (example : Alcatel Lucent@bob), or 2) external ID of user in VSD, typically is UUID generally decided by the CMS tool in questionUser needs to have CMS privileges to use proxy user header.
    """

    __rest_name__ = "container"
    __resource_name__ = "containers"

    
    ## Constants
    
    CONST_REASON_TYPE_SHUTDOWN_UNKNOWN = "SHUTDOWN_UNKNOWN"
    
    CONST_REASON_TYPE_CRASHED_UNKNOWN = "CRASHED_UNKNOWN"
    
    CONST_REASON_TYPE_PAUSED_IOERROR = "PAUSED_IOERROR"
    
    CONST_STATUS_SHUTDOWN = "SHUTDOWN"
    
    CONST_REASON_TYPE_SHUTDOWN_LAST = "SHUTDOWN_LAST"
    
    CONST_STATUS_DELETE_PENDING = "DELETE_PENDING"
    
    CONST_REASON_TYPE_RUNNING_UNKNOWN = "RUNNING_UNKNOWN"
    
    CONST_STATUS_RUNNING = "RUNNING"
    
    CONST_REASON_TYPE_RUNNING_LAST = "RUNNING_LAST"
    
    CONST_REASON_TYPE_RUNNING_UNPAUSED = "RUNNING_UNPAUSED"
    
    CONST_REASON_TYPE_PAUSED_FROM_SNAPSHOT = "PAUSED_FROM_SNAPSHOT"
    
    CONST_REASON_TYPE_PAUSED_MIGRATION = "PAUSED_MIGRATION"
    
    CONST_REASON_TYPE_RUNNING_BOOTED = "RUNNING_BOOTED"
    
    CONST_REASON_TYPE_UNKNOWN = "UNKNOWN"
    
    CONST_STATUS_UNREACHABLE = "UNREACHABLE"
    
    CONST_STATUS_BLOCKED = "BLOCKED"
    
    CONST_REASON_TYPE_SHUTOFF_DESTROYED = "SHUTOFF_DESTROYED"
    
    CONST_REASON_TYPE_SHUTOFF_FROM_SNAPSHOT = "SHUTOFF_FROM_SNAPSHOT"
    
    CONST_REASON_TYPE_SHUTOFF_UNKNOWN = "SHUTOFF_UNKNOWN"
    
    CONST_STATUS_NOSTATE = "NOSTATE"
    
    CONST_REASON_TYPE_PAUSED_DUMP = "PAUSED_DUMP"
    
    CONST_REASON_TYPE_CRASHED_LAST = "CRASHED_LAST"
    
    CONST_STATUS_CRASHED = "CRASHED"
    
    CONST_REASON_TYPE_PAUSED_LAST = "PAUSED_LAST"
    
    CONST_REASON_TYPE_BLOCKED_LAST = "BLOCKED_LAST"
    
    CONST_REASON_TYPE_SHUTOFF_LAST = "SHUTOFF_LAST"
    
    CONST_STATUS_SHUTOFF = "SHUTOFF"
    
    CONST_REASON_TYPE_SHUTOFF_SHUTDOWN = "SHUTOFF_SHUTDOWN"
    
    CONST_REASON_TYPE_NOSTATE_UNKNOWN = "NOSTATE_UNKNOWN"
    
    CONST_REASON_TYPE_PAUSED_SAVE = "PAUSED_SAVE"
    
    CONST_REASON_TYPE_RUNNING_FROM_SNAPSHOT = "RUNNING_FROM_SNAPSHOT"
    
    CONST_STATUS_UNKNOWN = "UNKNOWN"
    
    CONST_REASON_TYPE_PAUSED_UNKNOWN = "PAUSED_UNKNOWN"
    
    CONST_REASON_TYPE_SHUTOFF_FAILED = "SHUTOFF_FAILED"
    
    CONST_REASON_TYPE_SHUTOFF_SAVED = "SHUTOFF_SAVED"
    
    CONST_REASON_TYPE_SHUTOFF_MIGRATED = "SHUTOFF_MIGRATED"
    
    CONST_STATUS_LAST = "LAST"
    
    CONST_REASON_TYPE_RUNNING_MIGRATED = "RUNNING_MIGRATED"
    
    CONST_REASON_TYPE_RUNNING_SAVE_CANCELED = "RUNNING_SAVE_CANCELED"
    
    CONST_REASON_TYPE_SHUTDOWN_USER = "SHUTDOWN_USER"
    
    CONST_REASON_TYPE_RUNNING_MIGRATION_CANCELED = "RUNNING_MIGRATION_CANCELED"
    
    CONST_ENTITY_SCOPE_ENTERPRISE = "ENTERPRISE"
    
    CONST_STATUS_PAUSED = "PAUSED"
    
    CONST_STATUS_INIT = "INIT"
    
    CONST_REASON_TYPE_BLOCKED_UNKNOWN = "BLOCKED_UNKNOWN"
    
    CONST_REASON_TYPE_NOSTATE_LAST = "NOSTATE_LAST"
    
    CONST_REASON_TYPE_RUNNING_RESTORED = "RUNNING_RESTORED"
    
    CONST_ENTITY_SCOPE_GLOBAL = "GLOBAL"
    
    CONST_REASON_TYPE_SHUTOFF_CRASHED = "SHUTOFF_CRASHED"
    
    CONST_REASON_TYPE_PAUSED_USER = "PAUSED_USER"
    
    CONST_DELETE_MODE_TIMER = "TIMER"
    
    CONST_REASON_TYPE_PAUSED_WATCHDOG = "PAUSED_WATCHDOG"
    
    CONST_REASON_TYPE_PAUSED_SHUTTING_DOWN = "PAUSED_SHUTTING_DOWN"
    
    

    def __init__(self, **kwargs):
        """ Initializes a Container instance

            Notes:
                You can specify all parameters while calling this methods.
                A special argument named `data` will enable you to load the
                object from a Python dictionary

            Examples:
                >>> container = NUContainer(id=u'xxxx-xxx-xxx-xxx', name=u'Container')
                >>> container = NUContainer(data=my_dict)
        """

        super(NUContainer, self).__init__()

        # Read/Write Attributes
        
        self._l2_domain_ids = None
        self._vrsid = None
        self._uuid = None
        self._name = None
        self._last_updated_by = None
        self._reason_type = None
        self._delete_expiry = None
        self._delete_mode = None
        self._resync_info = None
        self._site_identifier = None
        self._image_id = None
        self._image_name = None
        self._interfaces = None
        self._enterprise_id = None
        self._enterprise_name = None
        self._entity_scope = None
        self._domain_ids = None
        self._compute_provisioned = None
        self._zone_ids = None
        self._orchestration_id = None
        self._user_id = None
        self._user_name = None
        self._status = None
        self._subnet_ids = None
        self._external_id = None
        self._hypervisor_ip = None
        
        self.expose_attribute(local_name="l2_domain_ids", remote_name="l2DomainIDs", attribute_type=list, is_required=False, is_unique=False)
        self.expose_attribute(local_name="vrsid", remote_name="VRSID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="uuid", remote_name="UUID", attribute_type=str, is_required=True, is_unique=False)
        self.expose_attribute(local_name="name", remote_name="name", attribute_type=str, is_required=True, is_unique=False)
        self.expose_attribute(local_name="last_updated_by", remote_name="lastUpdatedBy", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="reason_type", remote_name="reasonType", attribute_type=str, is_required=False, is_unique=False, choices=[u'BLOCKED_LAST', u'BLOCKED_UNKNOWN', u'CRASHED_LAST', u'CRASHED_UNKNOWN', u'NOSTATE_LAST', u'NOSTATE_UNKNOWN', u'PAUSED_DUMP', u'PAUSED_FROM_SNAPSHOT', u'PAUSED_IOERROR', u'PAUSED_LAST', u'PAUSED_MIGRATION', u'PAUSED_SAVE', u'PAUSED_SHUTTING_DOWN', u'PAUSED_UNKNOWN', u'PAUSED_USER', u'PAUSED_WATCHDOG', u'RUNNING_BOOTED', u'RUNNING_FROM_SNAPSHOT', u'RUNNING_LAST', u'RUNNING_MIGRATED', u'RUNNING_MIGRATION_CANCELED', u'RUNNING_RESTORED', u'RUNNING_SAVE_CANCELED', u'RUNNING_UNKNOWN', u'RUNNING_UNPAUSED', u'SHUTDOWN_LAST', u'SHUTDOWN_UNKNOWN', u'SHUTDOWN_USER', u'SHUTOFF_CRASHED', u'SHUTOFF_DESTROYED', u'SHUTOFF_FAILED', u'SHUTOFF_FROM_SNAPSHOT', u'SHUTOFF_LAST', u'SHUTOFF_MIGRATED', u'SHUTOFF_SAVED', u'SHUTOFF_SHUTDOWN', u'SHUTOFF_UNKNOWN', u'UNKNOWN'])
        self.expose_attribute(local_name="delete_expiry", remote_name="deleteExpiry", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="delete_mode", remote_name="deleteMode", attribute_type=str, is_required=False, is_unique=False, choices=[u'TIMER'])
        self.expose_attribute(local_name="resync_info", remote_name="resyncInfo", attribute_type=dict, is_required=False, is_unique=False)
        self.expose_attribute(local_name="site_identifier", remote_name="siteIdentifier", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="image_id", remote_name="imageID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="image_name", remote_name="imageName", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="interfaces", remote_name="interfaces", attribute_type=list, is_required=False, is_unique=False)
        self.expose_attribute(local_name="enterprise_id", remote_name="enterpriseID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="enterprise_name", remote_name="enterpriseName", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="entity_scope", remote_name="entityScope", attribute_type=str, is_required=False, is_unique=False, choices=[u'ENTERPRISE', u'GLOBAL'])
        self.expose_attribute(local_name="domain_ids", remote_name="domainIDs", attribute_type=list, is_required=False, is_unique=False)
        self.expose_attribute(local_name="compute_provisioned", remote_name="computeProvisioned", attribute_type=bool, is_required=False, is_unique=False)
        self.expose_attribute(local_name="zone_ids", remote_name="zoneIDs", attribute_type=list, is_required=False, is_unique=False)
        self.expose_attribute(local_name="orchestration_id", remote_name="orchestrationID", attribute_type=str, is_required=True, is_unique=False)
        self.expose_attribute(local_name="user_id", remote_name="userID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="user_name", remote_name="userName", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="status", remote_name="status", attribute_type=str, is_required=False, is_unique=False, choices=[u'BLOCKED', u'CRASHED', u'DELETE_PENDING', u'INIT', u'LAST', u'NOSTATE', u'PAUSED', u'RUNNING', u'SHUTDOWN', u'SHUTOFF', u'UNKNOWN', u'UNREACHABLE'])
        self.expose_attribute(local_name="subnet_ids", remote_name="subnetIDs", attribute_type=list, is_required=False, is_unique=False)
        self.expose_attribute(local_name="external_id", remote_name="externalID", attribute_type=str, is_required=False, is_unique=True)
        self.expose_attribute(local_name="hypervisor_ip", remote_name="hypervisorIP", attribute_type=str, is_required=False, is_unique=False)
        

        # Fetchers
        
        
        self.metadatas = NUMetadatasFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.alarms = NUAlarmsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.global_metadatas = NUGlobalMetadatasFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.container_interfaces = NUContainerInterfacesFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.container_resyncs = NUContainerResyncsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.vrss = NUVRSsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.event_logs = NUEventLogsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        

        self._compute_args(**kwargs)

    # Properties
    
    @property
    def l2_domain_ids(self):
        """ Get l2_domain_ids value.

            Notes:
                Array of IDs of the l2 domain that the container is connected to

                
                This attribute is named `l2DomainIDs` in VSD API.
                
        """
        return self._l2_domain_ids

    @l2_domain_ids.setter
    def l2_domain_ids(self, value):
        """ Set l2_domain_ids value.

            Notes:
                Array of IDs of the l2 domain that the container is connected to

                
                This attribute is named `l2DomainIDs` in VSD API.
                
        """
        self._l2_domain_ids = value

    
    @property
    def vrsid(self):
        """ Get vrsid value.

            Notes:
                Id of the VRS that this container is attached to.

                
                This attribute is named `VRSID` in VSD API.
                
        """
        return self._vrsid

    @vrsid.setter
    def vrsid(self, value):
        """ Set vrsid value.

            Notes:
                Id of the VRS that this container is attached to.

                
                This attribute is named `VRSID` in VSD API.
                
        """
        self._vrsid = value

    
    @property
    def uuid(self):
        """ Get uuid value.

            Notes:
                UUID of the container

                
                This attribute is named `UUID` in VSD API.
                
        """
        return self._uuid

    @uuid.setter
    def uuid(self, value):
        """ Set uuid value.

            Notes:
                UUID of the container

                
                This attribute is named `UUID` in VSD API.
                
        """
        self._uuid = value

    
    @property
    def name(self):
        """ Get name value.

            Notes:
                Name of the container

                
        """
        return self._name

    @name.setter
    def name(self, value):
        """ Set name value.

            Notes:
                Name of the container

                
        """
        self._name = value

    
    @property
    def last_updated_by(self):
        """ Get last_updated_by value.

            Notes:
                ID of the user who last updated the object.

                
                This attribute is named `lastUpdatedBy` in VSD API.
                
        """
        return self._last_updated_by

    @last_updated_by.setter
    def last_updated_by(self, value):
        """ Set last_updated_by value.

            Notes:
                ID of the user who last updated the object.

                
                This attribute is named `lastUpdatedBy` in VSD API.
                
        """
        self._last_updated_by = value

    
    @property
    def reason_type(self):
        """ Get reason_type value.

            Notes:
                Reason of the event associated with the container.

                
                This attribute is named `reasonType` in VSD API.
                
        """
        return self._reason_type

    @reason_type.setter
    def reason_type(self, value):
        """ Set reason_type value.

            Notes:
                Reason of the event associated with the container.

                
                This attribute is named `reasonType` in VSD API.
                
        """
        self._reason_type = value

    
    @property
    def delete_expiry(self):
        """ Get delete_expiry value.

            Notes:
                reflects the  container Deletion expiry timer in secs , deleteMode needs to be non-null value for deleteExpiry to be taken in to effect. CMS created containers will always have deleteMode set to TIMER

                
                This attribute is named `deleteExpiry` in VSD API.
                
        """
        return self._delete_expiry

    @delete_expiry.setter
    def delete_expiry(self, value):
        """ Set delete_expiry value.

            Notes:
                reflects the  container Deletion expiry timer in secs , deleteMode needs to be non-null value for deleteExpiry to be taken in to effect. CMS created containers will always have deleteMode set to TIMER

                
                This attribute is named `deleteExpiry` in VSD API.
                
        """
        self._delete_expiry = value

    
    @property
    def delete_mode(self):
        """ Get delete_mode value.

            Notes:
                reflects the mode of container Deletion -  TIMER  Possible values are TIMER, .

                
                This attribute is named `deleteMode` in VSD API.
                
        """
        return self._delete_mode

    @delete_mode.setter
    def delete_mode(self, value):
        """ Set delete_mode value.

            Notes:
                reflects the mode of container Deletion -  TIMER  Possible values are TIMER, .

                
                This attribute is named `deleteMode` in VSD API.
                
        """
        self._delete_mode = value

    
    @property
    def resync_info(self):
        """ Get resync_info value.

            Notes:
                Information of the status of the resync operation of a container

                
                This attribute is named `resyncInfo` in VSD API.
                
        """
        return self._resync_info

    @resync_info.setter
    def resync_info(self, value):
        """ Set resync_info value.

            Notes:
                Information of the status of the resync operation of a container

                
                This attribute is named `resyncInfo` in VSD API.
                
        """
        self._resync_info = value

    
    @property
    def site_identifier(self):
        """ Get site_identifier value.

            Notes:
                This property specifies the site the container belongs to, for Geo-redundancy.

                
                This attribute is named `siteIdentifier` in VSD API.
                
        """
        return self._site_identifier

    @site_identifier.setter
    def site_identifier(self, value):
        """ Set site_identifier value.

            Notes:
                This property specifies the site the container belongs to, for Geo-redundancy.

                
                This attribute is named `siteIdentifier` in VSD API.
                
        """
        self._site_identifier = value

    
    @property
    def image_id(self):
        """ Get image_id value.

            Notes:
                Id of the container image

                
                This attribute is named `imageID` in VSD API.
                
        """
        return self._image_id

    @image_id.setter
    def image_id(self, value):
        """ Set image_id value.

            Notes:
                Id of the container image

                
                This attribute is named `imageID` in VSD API.
                
        """
        self._image_id = value

    
    @property
    def image_name(self):
        """ Get image_name value.

            Notes:
                Name of the container image

                
                This attribute is named `imageName` in VSD API.
                
        """
        return self._image_name

    @image_name.setter
    def image_name(self, value):
        """ Set image_name value.

            Notes:
                Name of the container image

                
                This attribute is named `imageName` in VSD API.
                
        """
        self._image_name = value

    
    @property
    def interfaces(self):
        """ Get interfaces value.

            Notes:
                List of container interfaces associated with the container

                
        """
        return self._interfaces

    @interfaces.setter
    def interfaces(self, value):
        """ Set interfaces value.

            Notes:
                List of container interfaces associated with the container

                
        """
        self._interfaces = value

    
    @property
    def enterprise_id(self):
        """ Get enterprise_id value.

            Notes:
                ID of the enterprise that this container belongs to

                
                This attribute is named `enterpriseID` in VSD API.
                
        """
        return self._enterprise_id

    @enterprise_id.setter
    def enterprise_id(self, value):
        """ Set enterprise_id value.

            Notes:
                ID of the enterprise that this container belongs to

                
                This attribute is named `enterpriseID` in VSD API.
                
        """
        self._enterprise_id = value

    
    @property
    def enterprise_name(self):
        """ Get enterprise_name value.

            Notes:
                Name of the enterprise that this container belongs to

                
                This attribute is named `enterpriseName` in VSD API.
                
        """
        return self._enterprise_name

    @enterprise_name.setter
    def enterprise_name(self, value):
        """ Set enterprise_name value.

            Notes:
                Name of the enterprise that this container belongs to

                
                This attribute is named `enterpriseName` in VSD API.
                
        """
        self._enterprise_name = value

    
    @property
    def entity_scope(self):
        """ Get entity_scope value.

            Notes:
                Specify if scope of entity is Data center or Enterprise level

                
                This attribute is named `entityScope` in VSD API.
                
        """
        return self._entity_scope

    @entity_scope.setter
    def entity_scope(self, value):
        """ Set entity_scope value.

            Notes:
                Specify if scope of entity is Data center or Enterprise level

                
                This attribute is named `entityScope` in VSD API.
                
        """
        self._entity_scope = value

    
    @property
    def domain_ids(self):
        """ Get domain_ids value.

            Notes:
                Array of IDs of the domain that the container is connected to

                
                This attribute is named `domainIDs` in VSD API.
                
        """
        return self._domain_ids

    @domain_ids.setter
    def domain_ids(self, value):
        """ Set domain_ids value.

            Notes:
                Array of IDs of the domain that the container is connected to

                
                This attribute is named `domainIDs` in VSD API.
                
        """
        self._domain_ids = value

    
    @property
    def compute_provisioned(self):
        """ Get compute_provisioned value.

            Notes:
                Compute Provisioned

                
                This attribute is named `computeProvisioned` in VSD API.
                
        """
        return self._compute_provisioned

    @compute_provisioned.setter
    def compute_provisioned(self, value):
        """ Set compute_provisioned value.

            Notes:
                Compute Provisioned

                
                This attribute is named `computeProvisioned` in VSD API.
                
        """
        self._compute_provisioned = value

    
    @property
    def zone_ids(self):
        """ Get zone_ids value.

            Notes:
                Array of IDs of the zone that this container is attached to

                
                This attribute is named `zoneIDs` in VSD API.
                
        """
        return self._zone_ids

    @zone_ids.setter
    def zone_ids(self, value):
        """ Set zone_ids value.

            Notes:
                Array of IDs of the zone that this container is attached to

                
                This attribute is named `zoneIDs` in VSD API.
                
        """
        self._zone_ids = value

    
    @property
    def orchestration_id(self):
        """ Get orchestration_id value.

            Notes:
                Orchestration ID

                
                This attribute is named `orchestrationID` in VSD API.
                
        """
        return self._orchestration_id

    @orchestration_id.setter
    def orchestration_id(self, value):
        """ Set orchestration_id value.

            Notes:
                Orchestration ID

                
                This attribute is named `orchestrationID` in VSD API.
                
        """
        self._orchestration_id = value

    
    @property
    def user_id(self):
        """ Get user_id value.

            Notes:
                ID of the user that created this container

                
                This attribute is named `userID` in VSD API.
                
        """
        return self._user_id

    @user_id.setter
    def user_id(self, value):
        """ Set user_id value.

            Notes:
                ID of the user that created this container

                
                This attribute is named `userID` in VSD API.
                
        """
        self._user_id = value

    
    @property
    def user_name(self):
        """ Get user_name value.

            Notes:
                Username of the user that created this container

                
                This attribute is named `userName` in VSD API.
                
        """
        return self._user_name

    @user_name.setter
    def user_name(self, value):
        """ Set user_name value.

            Notes:
                Username of the user that created this container

                
                This attribute is named `userName` in VSD API.
                
        """
        self._user_name = value

    
    @property
    def status(self):
        """ Get status value.

            Notes:
                Status of the container.

                
        """
        return self._status

    @status.setter
    def status(self, value):
        """ Set status value.

            Notes:
                Status of the container.

                
        """
        self._status = value

    
    @property
    def subnet_ids(self):
        """ Get subnet_ids value.

            Notes:
                Array of IDs of the subnets that the container is connected to

                
                This attribute is named `subnetIDs` in VSD API.
                
        """
        return self._subnet_ids

    @subnet_ids.setter
    def subnet_ids(self, value):
        """ Set subnet_ids value.

            Notes:
                Array of IDs of the subnets that the container is connected to

                
                This attribute is named `subnetIDs` in VSD API.
                
        """
        self._subnet_ids = value

    
    @property
    def external_id(self):
        """ Get external_id value.

            Notes:
                External object ID. Used for integration with third party systems

                
                This attribute is named `externalID` in VSD API.
                
        """
        return self._external_id

    @external_id.setter
    def external_id(self, value):
        """ Set external_id value.

            Notes:
                External object ID. Used for integration with third party systems

                
                This attribute is named `externalID` in VSD API.
                
        """
        self._external_id = value

    
    @property
    def hypervisor_ip(self):
        """ Get hypervisor_ip value.

            Notes:
                IP address of the hypervisor that this container is currently running in

                
                This attribute is named `hypervisorIP` in VSD API.
                
        """
        return self._hypervisor_ip

    @hypervisor_ip.setter
    def hypervisor_ip(self, value):
        """ Set hypervisor_ip value.

            Notes:
                IP address of the hypervisor that this container is currently running in

                
                This attribute is named `hypervisorIP` in VSD API.
                
        """
        self._hypervisor_ip = value

    

    


# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc, 2017 Nokia
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.




from .fetchers import NUPermissionsFetcher


from .fetchers import NUMetadatasFetcher


from .fetchers import NUGlobalMetadatasFetcher


from .fetchers import NUVMsFetcher


from .fetchers import NUContainersFetcher


from .fetchers import NUGroupsFetcher


from .fetchers import NUAvatarsFetcher


from .fetchers import NUEventLogsFetcher

from bambou import NURESTObject


class NUUser(NURESTObject):
    """ Represents a User in the VSD

        Notes:
            Users represent people of your organization. A user can be placed into a group and this group can have some permissions to add VMs into a domain for instance.
    """

    __rest_name__ = "user"
    __resource_name__ = "users"

    
    ## Constants
    
    CONST_MANAGEMENT_MODE_CMS = "CMS"
    
    CONST_ENTITY_SCOPE_GLOBAL = "GLOBAL"
    
    CONST_ENTITY_SCOPE_ENTERPRISE = "ENTERPRISE"
    
    CONST_AVATAR_TYPE_URL = "URL"
    
    CONST_AVATAR_TYPE_COMPUTEDURL = "COMPUTEDURL"
    
    CONST_AVATAR_TYPE_BASE64 = "BASE64"
    
    CONST_MANAGEMENT_MODE_DEFAULT = "DEFAULT"
    
    

    def __init__(self, **kwargs):
        """ Initializes a User instance

            Notes:
                You can specify all parameters while calling this methods.
                A special argument named `data` will enable you to load the
                object from a Python dictionary

            Examples:
                >>> user = NUUser(id=u'xxxx-xxx-xxx-xxx', name=u'User')
                >>> user = NUUser(data=my_dict)
        """

        super(NUUser, self).__init__()

        # Read/Write Attributes
        
        self._ldapuser_dn = None
        self._management_mode = None
        self._password = None
        self._last_name = None
        self._last_updated_by = None
        self._last_updated_date = None
        self._first_name = None
        self._disable_certificate_auth = None
        self._disable_password_auth = None
        self._disabled = None
        self._email = None
        self._embedded_metadata = None
        self._entity_scope = None
        self._mobile_number = None
        self._creation_date = None
        self._user_name = None
        self._avatar_data = None
        self._avatar_type = None
        self._owner = None
        self._external_id = None
        
        self.expose_attribute(local_name="ldapuser_dn", remote_name="LDAPUserDN", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="management_mode", remote_name="managementMode", attribute_type=str, is_required=False, is_unique=False, choices=[u'CMS', u'DEFAULT'])
        self.expose_attribute(local_name="password", remote_name="password", attribute_type=str, is_required=True, is_unique=False)
        self.expose_attribute(local_name="last_name", remote_name="lastName", attribute_type=str, is_required=True, is_unique=False)
        self.expose_attribute(local_name="last_updated_by", remote_name="lastUpdatedBy", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="last_updated_date", remote_name="lastUpdatedDate", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="first_name", remote_name="firstName", attribute_type=str, is_required=True, is_unique=False)
        self.expose_attribute(local_name="disable_certificate_auth", remote_name="disableCertificateAuth", attribute_type=bool, is_required=False, is_unique=False)
        self.expose_attribute(local_name="disable_password_auth", remote_name="disablePasswordAuth", attribute_type=bool, is_required=False, is_unique=False)
        self.expose_attribute(local_name="disabled", remote_name="disabled", attribute_type=bool, is_required=False, is_unique=False)
        self.expose_attribute(local_name="email", remote_name="email", attribute_type=str, is_required=True, is_unique=False)
        self.expose_attribute(local_name="embedded_metadata", remote_name="embeddedMetadata", attribute_type=list, is_required=False, is_unique=False)
        self.expose_attribute(local_name="entity_scope", remote_name="entityScope", attribute_type=str, is_required=False, is_unique=False, choices=[u'ENTERPRISE', u'GLOBAL'])
        self.expose_attribute(local_name="mobile_number", remote_name="mobileNumber", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="creation_date", remote_name="creationDate", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="user_name", remote_name="userName", attribute_type=str, is_required=True, is_unique=False)
        self.expose_attribute(local_name="avatar_data", remote_name="avatarData", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="avatar_type", remote_name="avatarType", attribute_type=str, is_required=False, is_unique=False, choices=[u'BASE64', u'COMPUTEDURL', u'URL'])
        self.expose_attribute(local_name="owner", remote_name="owner", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="external_id", remote_name="externalID", attribute_type=str, is_required=False, is_unique=True)
        

        # Fetchers
        
        
        self.permissions = NUPermissionsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.metadatas = NUMetadatasFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.global_metadatas = NUGlobalMetadatasFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.vms = NUVMsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.containers = NUContainersFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.groups = NUGroupsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.avatars = NUAvatarsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.event_logs = NUEventLogsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        

        self._compute_args(**kwargs)

    # Properties
    
    @property
    def ldapuser_dn(self):
        """ Get ldapuser_dn value.

            Notes:
                The LDAP distinguished name (DN) for the user.

                
                This attribute is named `LDAPUserDN` in VSD API.
                
        """
        return self._ldapuser_dn

    @ldapuser_dn.setter
    def ldapuser_dn(self, value):
        """ Set ldapuser_dn value.

            Notes:
                The LDAP distinguished name (DN) for the user.

                
                This attribute is named `LDAPUserDN` in VSD API.
                
        """
        self._ldapuser_dn = value

    
    @property
    def management_mode(self):
        """ Get management_mode value.

            Notes:
                Management mode of the user object - allows for override of external authorization and syncup

                
                This attribute is named `managementMode` in VSD API.
                
        """
        return self._management_mode

    @management_mode.setter
    def management_mode(self, value):
        """ Set management_mode value.

            Notes:
                Management mode of the user object - allows for override of external authorization and syncup

                
                This attribute is named `managementMode` in VSD API.
                
        """
        self._management_mode = value

    
    @property
    def password(self):
        """ Get password value.

            Notes:
                User password in clear text. Password cannot be a single character asterisk (*)

                
        """
        return self._password

    @password.setter
    def password(self, value):
        """ Set password value.

            Notes:
                User password in clear text. Password cannot be a single character asterisk (*)

                
        """
        self._password = value

    
    @property
    def last_name(self):
        """ Get last_name value.

            Notes:
                Last name of the user

                
                This attribute is named `lastName` in VSD API.
                
        """
        return self._last_name

    @last_name.setter
    def last_name(self, value):
        """ Set last_name value.

            Notes:
                Last name of the user

                
                This attribute is named `lastName` in VSD API.
                
        """
        self._last_name = value

    
    @property
    def last_updated_by(self):
        """ Get last_updated_by value.

            Notes:
                ID of the user who last updated the object.

                
                This attribute is named `lastUpdatedBy` in VSD API.
                
        """
        return self._last_updated_by

    @last_updated_by.setter
    def last_updated_by(self, value):
        """ Set last_updated_by value.

            Notes:
                ID of the user who last updated the object.

                
                This attribute is named `lastUpdatedBy` in VSD API.
                
        """
        self._last_updated_by = value

    
    @property
    def last_updated_date(self):
        """ Get last_updated_date value.

            Notes:
                Time stamp when this object was last updated.

                
                This attribute is named `lastUpdatedDate` in VSD API.
                
        """
        return self._last_updated_date

    @last_updated_date.setter
    def last_updated_date(self, value):
        """ Set last_updated_date value.

            Notes:
                Time stamp when this object was last updated.

                
                This attribute is named `lastUpdatedDate` in VSD API.
                
        """
        self._last_updated_date = value

    
    @property
    def first_name(self):
        """ Get first_name value.

            Notes:
                First name of the user

                
                This attribute is named `firstName` in VSD API.
                
        """
        return self._first_name

    @first_name.setter
    def first_name(self, value):
        """ Set first_name value.

            Notes:
                First name of the user

                
                This attribute is named `firstName` in VSD API.
                
        """
        self._first_name = value

    
    @property
    def disable_certificate_auth(self):
        """ Get disable_certificate_auth value.

            Notes:
                Whether Certificate-Based Authentication is disabled. Default is false.

                
                This attribute is named `disableCertificateAuth` in VSD API.
                
        """
        return self._disable_certificate_auth

    @disable_certificate_auth.setter
    def disable_certificate_auth(self, value):
        """ Set disable_certificate_auth value.

            Notes:
                Whether Certificate-Based Authentication is disabled. Default is false.

                
                This attribute is named `disableCertificateAuth` in VSD API.
                
        """
        self._disable_certificate_auth = value

    
    @property
    def disable_password_auth(self):
        """ Get disable_password_auth value.

            Notes:
                Whether Username-Password Authentication is disabled. Default is false.

                
                This attribute is named `disablePasswordAuth` in VSD API.
                
        """
        return self._disable_password_auth

    @disable_password_auth.setter
    def disable_password_auth(self, value):
        """ Set disable_password_auth value.

            Notes:
                Whether Username-Password Authentication is disabled. Default is false.

                
                This attribute is named `disablePasswordAuth` in VSD API.
                
        """
        self._disable_password_auth = value

    
    @property
    def disabled(self):
        """ Get disabled value.

            Notes:
                Status of the user account; true=disabled, false=not disabled; default value = false

                
        """
        return self._disabled

    @disabled.setter
    def disabled(self, value):
        """ Set disabled value.

            Notes:
                Status of the user account; true=disabled, false=not disabled; default value = false

                
        """
        self._disabled = value

    
    @property
    def email(self):
        """ Get email value.

            Notes:
                Email address of the user

                
        """
        return self._email

    @email.setter
    def email(self, value):
        """ Set email value.

            Notes:
                Email address of the user

                
        """
        self._email = value

    
    @property
    def embedded_metadata(self):
        """ Get embedded_metadata value.

            Notes:
                Metadata objects associated with this entity. This will contain a list of Metadata objects if the API request is made using the special flag to enable the embedded Metadata feature. Only a maximum of Metadata objects is returned based on the value set in the system configuration.

                
                This attribute is named `embeddedMetadata` in VSD API.
                
        """
        return self._embedded_metadata

    @embedded_metadata.setter
    def embedded_metadata(self, value):
        """ Set embedded_metadata value.

            Notes:
                Metadata objects associated with this entity. This will contain a list of Metadata objects if the API request is made using the special flag to enable the embedded Metadata feature. Only a maximum of Metadata objects is returned based on the value set in the system configuration.

                
                This attribute is named `embeddedMetadata` in VSD API.
                
        """
        self._embedded_metadata = value

    
    @property
    def entity_scope(self):
        """ Get entity_scope value.

            Notes:
                Specify if scope of entity is Data center or Enterprise level

                
                This attribute is named `entityScope` in VSD API.
                
        """
        return self._entity_scope

    @entity_scope.setter
    def entity_scope(self, value):
        """ Set entity_scope value.

            Notes:
                Specify if scope of entity is Data center or Enterprise level

                
                This attribute is named `entityScope` in VSD API.
                
        """
        self._entity_scope = value

    
    @property
    def mobile_number(self):
        """ Get mobile_number value.

            Notes:
                Mobile Number of the user

                
                This attribute is named `mobileNumber` in VSD API.
                
        """
        return self._mobile_number

    @mobile_number.setter
    def mobile_number(self, value):
        """ Set mobile_number value.

            Notes:
                Mobile Number of the user

                
                This attribute is named `mobileNumber` in VSD API.
                
        """
        self._mobile_number = value

    
    @property
    def creation_date(self):
        """ Get creation_date value.

            Notes:
                Time stamp when this object was created.

                
                This attribute is named `creationDate` in VSD API.
                
        """
        return self._creation_date

    @creation_date.setter
    def creation_date(self, value):
        """ Set creation_date value.

            Notes:
                Time stamp when this object was created.

                
                This attribute is named `creationDate` in VSD API.
                
        """
        self._creation_date = value

    
    @property
    def user_name(self):
        """ Get user_name value.

            Notes:
                Unique Username of the user. Valid characters are alphabets, numbers and hyphen( - ).

                
                This attribute is named `userName` in VSD API.
                
        """
        return self._user_name

    @user_name.setter
    def user_name(self, value):
        """ Set user_name value.

            Notes:
                Unique Username of the user. Valid characters are alphabets, numbers and hyphen( - ).

                
                This attribute is named `userName` in VSD API.
                
        """
        self._user_name = value

    
    @property
    def avatar_data(self):
        """ Get avatar_data value.

            Notes:
                URL to the avatar data associated with the enterprise. If the avatarType is URL then value of avatarData should an URL of the image. If the avatarType BASE64 then avatarData should be BASE64 encoded value of the image

                
                This attribute is named `avatarData` in VSD API.
                
        """
        return self._avatar_data

    @avatar_data.setter
    def avatar_data(self, value):
        """ Set avatar_data value.

            Notes:
                URL to the avatar data associated with the enterprise. If the avatarType is URL then value of avatarData should an URL of the image. If the avatarType BASE64 then avatarData should be BASE64 encoded value of the image

                
                This attribute is named `avatarData` in VSD API.
                
        """
        self._avatar_data = value

    
    @property
    def avatar_type(self):
        """ Get avatar_type value.

            Notes:
                Avatar type.

                
                This attribute is named `avatarType` in VSD API.
                
        """
        return self._avatar_type

    @avatar_type.setter
    def avatar_type(self, value):
        """ Set avatar_type value.

            Notes:
                Avatar type.

                
                This attribute is named `avatarType` in VSD API.
                
        """
        self._avatar_type = value

    
    @property
    def owner(self):
        """ Get owner value.

            Notes:
                Identifies the user that has created this object.

                
        """
        return self._owner

    @owner.setter
    def owner(self, value):
        """ Set owner value.

            Notes:
                Identifies the user that has created this object.

                
        """
        self._owner = value

    
    @property
    def external_id(self):
        """ Get external_id value.

            Notes:
                External object ID. Used for integration with third party systems

                
                This attribute is named `externalID` in VSD API.
                
        """
        return self._external_id

    @external_id.setter
    def external_id(self, value):
        """ Set external_id value.

            Notes:
                External object ID. Used for integration with third party systems

                
                This attribute is named `externalID` in VSD API.
                
        """
        self._external_id = value

    

    


# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc, 2017 Nokia
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.




from .fetchers import NUMetadatasFetcher


from .fetchers import NUAlarmsFetcher


from .fetchers import NUGlobalMetadatasFetcher


from .fetchers import NUEventLogsFetcher

from bambou import NURESTObject


class NUTCA(NURESTObject):
    """ Represents a TCA in the VSD

        Notes:
            Provides the definition of the Threshold Control Alarms.
    """

    __rest_name__ = "tca"
    __resource_name__ = "tcas"

    
    ## Constants
    
    CONST_METRIC_PACKETS_OUT_DROPPED = "PACKETS_OUT_DROPPED"
    
    CONST_METRIC_Q4_BYTES = "Q4_BYTES"
    
    CONST_METRIC_FIP_RATE_LIMIT_DROPPED_PACKETS = "FIP_RATE_LIMIT_DROPPED_PACKETS"
    
    CONST_METRIC_TX_ERRORS = "TX_ERRORS"
    
    CONST_METRIC_RX_BYTES = "RX_BYTES"
    
    CONST_TYPE_BREACH = "BREACH"
    
    CONST_ENTITY_SCOPE_ENTERPRISE = "ENTERPRISE"
    
    CONST_METRIC_TCP_FLAG_RST_IN = "TCP_FLAG_RST_IN"
    
    CONST_METRIC_RX_DROPPED = "RX_DROPPED"
    
    CONST_METRIC_Q3_PKT_COUNT = "Q3_PKT_COUNT"
    
    CONST_METRIC_TCP_FLAG_ACK_OUT = "TCP_FLAG_ACK_OUT"
    
    CONST_METRIC_TCP_FLAG_ACK_IN = "TCP_FLAG_ACK_IN"
    
    CONST_METRIC_EGRESS_PACKET_COUNT = "EGRESS_PACKET_COUNT"
    
    CONST_METRIC_Q2_DROPPED = "Q2_DROPPED"
    
    CONST_METRIC_PACKETS_IN_DROPPED = "PACKETS_IN_DROPPED"
    
    CONST_METRIC_L7_BYTES_OUT = "L7_BYTES_OUT"
    
    CONST_TYPE_ROLLING_AVERAGE = "ROLLING_AVERAGE"
    
    CONST_METRIC_FIP_RATE_LIMIT_DROPPED_BYTES = "FIP_RATE_LIMIT_DROPPED_BYTES"
    
    CONST_METRIC_PACKETS_DROPPED_BY_RATE_LIMIT = "PACKETS_DROPPED_BY_RATE_LIMIT"
    
    CONST_METRIC_Q1_BYTES = "Q1_BYTES"
    
    CONST_METRIC_Q3_BYTES = "Q3_BYTES"
    
    CONST_METRIC_EGRESS_BYTE_COUNT = "EGRESS_BYTE_COUNT"
    
    CONST_METRIC_INGRESS_PACKET_COUNT = "INGRESS_PACKET_COUNT"
    
    CONST_METRIC_TX_DROPPED = "TX_DROPPED"
    
    CONST_METRIC_Q1_DROPPED = "Q1_DROPPED"
    
    CONST_METRIC_ADDRESS_MAP_INGRESS_BYTE_CNT = "ADDRESS_MAP_INGRESS_BYTE_CNT"
    
    CONST_METRIC_PACKETS_IN = "PACKETS_IN"
    
    CONST_METRIC_Q2_PKT_COUNT = "Q2_PKT_COUNT"
    
    CONST_METRIC_RX_PKT_COUNT = "RX_PKT_COUNT"
    
    CONST_METRIC_TX_BYTES = "TX_BYTES"
    
    CONST_METRIC_ANTI_SPOOF_EVENT_COUNT = "ANTI_SPOOF_EVENT_COUNT"
    
    CONST_METRIC_ACL_DENY_EVENT_COUNT = "ACL_DENY_EVENT_COUNT"
    
    CONST_METRIC_TCP_FLAG_NULL_IN = "TCP_FLAG_NULL_IN"
    
    CONST_METRIC_Q3_DROPPED = "Q3_DROPPED"
    
    CONST_METRIC_TCP_FLAG_NULL_OUT = "TCP_FLAG_NULL_OUT"
    
    CONST_METRIC_Q10_BYTES = "Q10_BYTES"
    
    CONST_METRIC_BYTES_IN = "BYTES_IN"
    
    CONST_METRIC_Q10_PKT_COUNT = "Q10_PKT_COUNT"
    
    CONST_METRIC_FIP_PRE_RATE_LIMIT_BYTES = "FIP_PRE_RATE_LIMIT_BYTES"
    
    CONST_METRIC_Q4_PKT_COUNT = "Q4_PKT_COUNT"
    
    CONST_METRIC_ADDRESS_MAP_EGRESS_BYTE_CNT = "ADDRESS_MAP_EGRESS_BYTE_CNT"
    
    CONST_METRIC_INGRESS_BYTE_COUNT = "INGRESS_BYTE_COUNT"
    
    CONST_ACTION_ALERT_POLICYGROUPCHANGE = "Alert_PolicyGroupChange"
    
    CONST_METRIC_PACKETS_IN_ERROR = "PACKETS_IN_ERROR"
    
    CONST_METRIC_TX_PKT_COUNT = "TX_PKT_COUNT"
    
    CONST_METRIC_TCP_FLAG_RST_OUT = "TCP_FLAG_RST_OUT"
    
    CONST_METRIC_Q1_PKT_COUNT = "Q1_PKT_COUNT"
    
    CONST_METRIC_ADDRESS_MAP_EGRESS_PKT_CNT = "ADDRESS_MAP_EGRESS_PKT_CNT"
    
    CONST_METRIC_ADDRESS_MAP_INGRESS_PKT_CNT = "ADDRESS_MAP_INGRESS_PKT_CNT"
    
    CONST_METRIC_RX_ERRORS = "RX_ERRORS"
    
    CONST_METRIC_Q0_BYTES = "Q0_BYTES"
    
    CONST_METRIC_PACKETS_OUT_ERROR = "PACKETS_OUT_ERROR"
    
    CONST_METRIC_BYTES_OUT = "BYTES_OUT"
    
    CONST_METRIC_L7_PACKETS_OUT = "L7_PACKETS_OUT"
    
    CONST_METRIC_Q0_PKT_COUNT = "Q0_PKT_COUNT"
    
    CONST_METRIC_L7_BYTES_IN = "L7_BYTES_IN"
    
    CONST_ENTITY_SCOPE_GLOBAL = "GLOBAL"
    
    CONST_METRIC_TCP_FLAG_SYN_OUT = "TCP_FLAG_SYN_OUT"
    
    CONST_METRIC_PACKETS_OUT = "PACKETS_OUT"
    
    CONST_METRIC_Q10_DROPPED = "Q10_DROPPED"
    
    CONST_METRIC_Q4_DROPPED = "Q4_DROPPED"
    
    CONST_METRIC_CONNECTION_TYPE = "CONNECTION_TYPE"
    
    CONST_METRIC_Q2_BYTES = "Q2_BYTES"
    
    CONST_METRIC_FIP_PRE_RATE_LIMIT_PACKETS = "FIP_PRE_RATE_LIMIT_PACKETS"
    
    CONST_ACTION_ALERT = "Alert"
    
    CONST_METRIC_TCP_FLAG_SYN_IN = "TCP_FLAG_SYN_IN"
    
    CONST_METRIC_Q0_DROPPED = "Q0_DROPPED"
    
    CONST_METRIC_TCP_SYN_EVENT_COUNT = "TCP_SYN_EVENT_COUNT"
    
    CONST_METRIC_L7_PACKETS_IN = "L7_PACKETS_IN"
    
    

    def __init__(self, **kwargs):
        """ Initializes a TCA instance

            Notes:
                You can specify all parameters while calling this methods.
                A special argument named `data` will enable you to load the
                object from a Python dictionary

            Examples:
                >>> tca = NUTCA(id=u'xxxx-xxx-xxx-xxx', name=u'TCA')
                >>> tca = NUTCA(data=my_dict)
        """

        super(NUTCA, self).__init__()

        # Read/Write Attributes
        
        self._url_end_point = None
        self._name = None
        self._target_policy_group_id = None
        self._last_updated_by = None
        self._action = None
        self._period = None
        self._description = None
        self._metric = None
        self._threshold = None
        self._throttle_time = None
        self._disable = None
        self._display_status = None
        self._entity_scope = None
        self._count = None
        self._status = None
        self._external_id = None
        self._type = None
        
        self.expose_attribute(local_name="url_end_point", remote_name="URLEndPoint", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="name", remote_name="name", attribute_type=str, is_required=True, is_unique=False)
        self.expose_attribute(local_name="target_policy_group_id", remote_name="targetPolicyGroupID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="last_updated_by", remote_name="lastUpdatedBy", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="action", remote_name="action", attribute_type=str, is_required=True, is_unique=False, choices=[u'Alert', u'Alert_PolicyGroupChange'])
        self.expose_attribute(local_name="period", remote_name="period", attribute_type=int, is_required=True, is_unique=False)
        self.expose_attribute(local_name="description", remote_name="description", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="metric", remote_name="metric", attribute_type=str, is_required=True, is_unique=False, choices=[u'ACL_DENY_EVENT_COUNT', u'ADDRESS_MAP_EGRESS_BYTE_CNT', u'ADDRESS_MAP_EGRESS_PKT_CNT', u'ADDRESS_MAP_INGRESS_BYTE_CNT', u'ADDRESS_MAP_INGRESS_PKT_CNT', u'ANTI_SPOOF_EVENT_COUNT', u'BYTES_IN', u'BYTES_OUT', u'CONNECTION_TYPE', u'EGRESS_BYTE_COUNT', u'EGRESS_PACKET_COUNT', u'FIP_PRE_RATE_LIMIT_BYTES', u'FIP_PRE_RATE_LIMIT_PACKETS', u'FIP_RATE_LIMIT_DROPPED_BYTES', u'FIP_RATE_LIMIT_DROPPED_PACKETS', u'INGRESS_BYTE_COUNT', u'INGRESS_PACKET_COUNT', u'L7_BYTES_IN', u'L7_BYTES_OUT', u'L7_PACKETS_IN', u'L7_PACKETS_OUT', u'PACKETS_DROPPED_BY_RATE_LIMIT', u'PACKETS_IN', u'PACKETS_IN_DROPPED', u'PACKETS_IN_ERROR', u'PACKETS_OUT', u'PACKETS_OUT_DROPPED', u'PACKETS_OUT_ERROR', u'Q0_BYTES', u'Q0_DROPPED', u'Q0_PKT_COUNT', u'Q10_BYTES', u'Q10_DROPPED', u'Q10_PKT_COUNT', u'Q1_BYTES', u'Q1_DROPPED', u'Q1_PKT_COUNT', u'Q2_BYTES', u'Q2_DROPPED', u'Q2_PKT_COUNT', u'Q3_BYTES', u'Q3_DROPPED', u'Q3_PKT_COUNT', u'Q4_BYTES', u'Q4_DROPPED', u'Q4_PKT_COUNT', u'RX_BYTES', u'RX_DROPPED', u'RX_ERRORS', u'RX_PKT_COUNT', u'TCP_FLAG_ACK_IN', u'TCP_FLAG_ACK_OUT', u'TCP_FLAG_NULL_IN', u'TCP_FLAG_NULL_OUT', u'TCP_FLAG_RST_IN', u'TCP_FLAG_RST_OUT', u'TCP_FLAG_SYN_IN', u'TCP_FLAG_SYN_OUT', u'TCP_SYN_EVENT_COUNT', u'TX_BYTES', u'TX_DROPPED', u'TX_ERRORS', u'TX_PKT_COUNT'])
        self.expose_attribute(local_name="threshold", remote_name="threshold", attribute_type=int, is_required=True, is_unique=False)
        self.expose_attribute(local_name="throttle_time", remote_name="throttleTime", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="disable", remote_name="disable", attribute_type=bool, is_required=False, is_unique=False)
        self.expose_attribute(local_name="display_status", remote_name="displayStatus", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="entity_scope", remote_name="entityScope", attribute_type=str, is_required=False, is_unique=False, choices=[u'ENTERPRISE', u'GLOBAL'])
        self.expose_attribute(local_name="count", remote_name="count", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="status", remote_name="status", attribute_type=bool, is_required=False, is_unique=False)
        self.expose_attribute(local_name="external_id", remote_name="externalID", attribute_type=str, is_required=False, is_unique=True)
        self.expose_attribute(local_name="type", remote_name="type", attribute_type=str, is_required=True, is_unique=False, choices=[u'BREACH', u'ROLLING_AVERAGE'])
        

        # Fetchers
        
        
        self.metadatas = NUMetadatasFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.alarms = NUAlarmsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.global_metadatas = NUGlobalMetadatasFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.event_logs = NUEventLogsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        

        self._compute_args(**kwargs)

    # Properties
    
    @property
    def url_end_point(self):
        """ Get url_end_point value.

            Notes:
                URL endpoint to post Alarm data to when TCA is triggered

                
                This attribute is named `URLEndPoint` in VSD API.
                
        """
        return self._url_end_point

    @url_end_point.setter
    def url_end_point(self, value):
        """ Set url_end_point value.

            Notes:
                URL endpoint to post Alarm data to when TCA is triggered

                
                This attribute is named `URLEndPoint` in VSD API.
                
        """
        self._url_end_point = value

    
    @property
    def name(self):
        """ Get name value.

            Notes:
                The name of the TCA

                
        """
        return self._name

    @name.setter
    def name(self, value):
        """ Set name value.

            Notes:
                The name of the TCA

                
        """
        self._name = value

    
    @property
    def target_policy_group_id(self):
        """ Get target_policy_group_id value.

            Notes:
                Target policygroup when TCA is triggered

                
                This attribute is named `targetPolicyGroupID` in VSD API.
                
        """
        return self._target_policy_group_id

    @target_policy_group_id.setter
    def target_policy_group_id(self, value):
        """ Set target_policy_group_id value.

            Notes:
                Target policygroup when TCA is triggered

                
                This attribute is named `targetPolicyGroupID` in VSD API.
                
        """
        self._target_policy_group_id = value

    
    @property
    def last_updated_by(self):
        """ Get last_updated_by value.

            Notes:
                ID of the user who last updated the object.

                
                This attribute is named `lastUpdatedBy` in VSD API.
                
        """
        return self._last_updated_by

    @last_updated_by.setter
    def last_updated_by(self, value):
        """ Set last_updated_by value.

            Notes:
                ID of the user who last updated the object.

                
                This attribute is named `lastUpdatedBy` in VSD API.
                
        """
        self._last_updated_by = value

    
    @property
    def action(self):
        """ Get action value.

            Notes:
                Action to be taken when TCA is fired - Alert or PolicyGroupChange

                
        """
        return self._action

    @action.setter
    def action(self, value):
        """ Set action value.

            Notes:
                Action to be taken when TCA is fired - Alert or PolicyGroupChange

                
        """
        self._action = value

    
    @property
    def period(self):
        """ Get period value.

            Notes:
                The averaging period

                
        """
        return self._period

    @period.setter
    def period(self, value):
        """ Set period value.

            Notes:
                The averaging period

                
        """
        self._period = value

    
    @property
    def description(self):
        """ Get description value.

            Notes:
                Description of the TCA

                
        """
        return self._description

    @description.setter
    def description(self, value):
        """ Set description value.

            Notes:
                Description of the TCA

                
        """
        self._description = value

    
    @property
    def metric(self):
        """ Get metric value.

            Notes:
                The metric associated with the TCA. The following enum values have been deprecated and will be removed in the next major release 6.0: ADDRESS_MAP_EGRESS_BYTE_CNT, ADDRESS_MAP_EGRESS_PKT_CNT, ADDRESS_MAP_INGRESS_BYTE_CNT, ADDRESS_MAP_INGRESS_PKT_CNT, CONNECTION_TYPE, EGRESS_BYTE_COUNT, EGRESS_PACKET_COUNT, INGRESS_BYTE_COUNT, INGRESS_PACKET_COUNT, Q0_BYTES, Q0_DROPPED, Q0_PKT_COUNT, Q10_BYTES, Q10_DROPPED, Q10_PKT_COUNT, Q1_BYTES, Q1_DROPPED, Q1_PKT_COUNT, Q2_BYTES, Q2_DROPPED, Q2_PKT_COUNT, Q3_BYTES, Q3_DROPPED, Q3_PKT_COUNT, Q4_BYTES, Q4_DROPPED, Q4_PKT_COUNT, RX_BYTES, RX_DROPPED, RX_ERRORS, RX_PKT_COUNT, TCP_SYN_EVENT_COUNT, TX_BYTES, TX_DROPPED, TX_ERRORS, TX_PKT_COUNT

                
        """
        return self._metric

    @metric.setter
    def metric(self, value):
        """ Set metric value.

            Notes:
                The metric associated with the TCA. The following enum values have been deprecated and will be removed in the next major release 6.0: ADDRESS_MAP_EGRESS_BYTE_CNT, ADDRESS_MAP_EGRESS_PKT_CNT, ADDRESS_MAP_INGRESS_BYTE_CNT, ADDRESS_MAP_INGRESS_PKT_CNT, CONNECTION_TYPE, EGRESS_BYTE_COUNT, EGRESS_PACKET_COUNT, INGRESS_BYTE_COUNT, INGRESS_PACKET_COUNT, Q0_BYTES, Q0_DROPPED, Q0_PKT_COUNT, Q10_BYTES, Q10_DROPPED, Q10_PKT_COUNT, Q1_BYTES, Q1_DROPPED, Q1_PKT_COUNT, Q2_BYTES, Q2_DROPPED, Q2_PKT_COUNT, Q3_BYTES, Q3_DROPPED, Q3_PKT_COUNT, Q4_BYTES, Q4_DROPPED, Q4_PKT_COUNT, RX_BYTES, RX_DROPPED, RX_ERRORS, RX_PKT_COUNT, TCP_SYN_EVENT_COUNT, TX_BYTES, TX_DROPPED, TX_ERRORS, TX_PKT_COUNT

                
        """
        self._metric = value

    
    @property
    def threshold(self):
        """ Get threshold value.

            Notes:
                The threshold that must be exceeded before an alarm is issued

                
        """
        return self._threshold

    @threshold.setter
    def threshold(self, value):
        """ Set threshold value.

            Notes:
                The threshold that must be exceeded before an alarm is issued

                
        """
        self._threshold = value

    
    @property
    def throttle_time(self):
        """ Get throttle_time value.

            Notes:
                Throttle time in secs

                
                This attribute is named `throttleTime` in VSD API.
                
        """
        return self._throttle_time

    @throttle_time.setter
    def throttle_time(self, value):
        """ Set throttle_time value.

            Notes:
                Throttle time in secs

                
                This attribute is named `throttleTime` in VSD API.
                
        """
        self._throttle_time = value

    
    @property
    def disable(self):
        """ Get disable value.

            Notes:
                This flag is used to indicate whether the watch(TCA) is enabled/disabled

                
        """
        return self._disable

    @disable.setter
    def disable(self, value):
        """ Set disable value.

            Notes:
                This flag is used to indicate whether the watch(TCA) is enabled/disabled

                
        """
        self._disable = value

    
    @property
    def display_status(self):
        """ Get display_status value.

            Notes:
                Explanation of the TCA status

                
                This attribute is named `displayStatus` in VSD API.
                
        """
        return self._display_status

    @display_status.setter
    def display_status(self, value):
        """ Set display_status value.

            Notes:
                Explanation of the TCA status

                
                This attribute is named `displayStatus` in VSD API.
                
        """
        self._display_status = value

    
    @property
    def entity_scope(self):
        """ Get entity_scope value.

            Notes:
                Specify if scope of entity is Data center or Enterprise level

                
                This attribute is named `entityScope` in VSD API.
                
        """
        return self._entity_scope

    @entity_scope.setter
    def entity_scope(self, value):
        """ Set entity_scope value.

            Notes:
                Specify if scope of entity is Data center or Enterprise level

                
                This attribute is named `entityScope` in VSD API.
                
        """
        self._entity_scope = value

    
    @property
    def count(self):
        """ Get count value.

            Notes:
                Count of the attempts by maintenanace thread to create/update watcher

                
        """
        return self._count

    @count.setter
    def count(self, value):
        """ Set count value.

            Notes:
                Count of the attempts by maintenanace thread to create/update watcher

                
        """
        self._count = value

    
    @property
    def status(self):
        """ Get status value.

            Notes:
                This flag is used to indicate the status of TCA

                
        """
        return self._status

    @status.setter
    def status(self, value):
        """ Set status value.

            Notes:
                This flag is used to indicate the status of TCA

                
        """
        self._status = value

    
    @property
    def external_id(self):
        """ Get external_id value.

            Notes:
                External object ID. Used for integration with third party systems

                
                This attribute is named `externalID` in VSD API.
                
        """
        return self._external_id

    @external_id.setter
    def external_id(self, value):
        """ Set external_id value.

            Notes:
                External object ID. Used for integration with third party systems

                
                This attribute is named `externalID` in VSD API.
                
        """
        self._external_id = value

    
    @property
    def type(self):
        """ Get type value.

            Notes:
                Rolling average or sequence of samples over the averaging period.

                
        """
        return self._type

    @type.setter
    def type(self, value):
        """ Set type value.

            Notes:
                Rolling average or sequence of samples over the averaging period.

                
        """
        self._type = value

    

    


# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc, 2017 Nokia
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.




from .fetchers import NUMetadatasFetcher


from .fetchers import NUGlobalMetadatasFetcher


from .fetchers import NUOSPFInterfacesFetcher

from bambou import NURESTObject


class NUOSPFArea(NURESTObject):
    """ Represents a OSPFArea in the VSD

        Notes:
            OSPF relies on the concept of logical areas. The use of areas enables the hiding of topology information between areas whilst still providing reachability. Each router in the area shares the same routing tables, which simplifies the network topology and helps to optimize the route calculation algorithm. 
    """

    __rest_name__ = "ospfarea"
    __resource_name__ = "ospfareas"

    
    ## Constants
    
    CONST_ENTITY_SCOPE_GLOBAL = "GLOBAL"
    
    CONST_DEFAULT_ORIGINATE_OPTION_NONE = "NONE"
    
    CONST_ENTITY_SCOPE_ENTERPRISE = "ENTERPRISE"
    
    CONST_AREA_TYPE_NORMAL = "NORMAL"
    
    CONST_AREA_TYPE_NSSA = "NSSA"
    
    CONST_DEFAULT_ORIGINATE_OPTION_TYPE3 = "TYPE3"
    
    CONST_AREA_TYPE_STUB = "STUB"
    
    CONST_DEFAULT_ORIGINATE_OPTION_TYPE7 = "TYPE7"
    
    

    def __init__(self, **kwargs):
        """ Initializes a OSPFArea instance

            Notes:
                You can specify all parameters while calling this methods.
                A special argument named `data` will enable you to load the
                object from a Python dictionary

            Examples:
                >>> ospfarea = NUOSPFArea(id=u'xxxx-xxx-xxx-xxx', name=u'OSPFArea')
                >>> ospfarea = NUOSPFArea(data=my_dict)
        """

        super(NUOSPFArea, self).__init__()

        # Read/Write Attributes
        
        self._last_updated_by = None
        self._redistribute_external_enabled = None
        self._default_metric = None
        self._default_originate_option = None
        self._description = None
        self._aggregate_area_range = None
        self._aggregate_area_range_nssa = None
        self._entity_scope = None
        self._area_id = None
        self._area_type = None
        self._summaries_enabled = None
        self._suppress_area_range = None
        self._suppress_area_range_nssa = None
        self._external_id = None
        
        self.expose_attribute(local_name="last_updated_by", remote_name="lastUpdatedBy", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="redistribute_external_enabled", remote_name="redistributeExternalEnabled", attribute_type=bool, is_required=False, is_unique=False)
        self.expose_attribute(local_name="default_metric", remote_name="defaultMetric", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="default_originate_option", remote_name="defaultOriginateOption", attribute_type=str, is_required=False, is_unique=False, choices=[u'NONE', u'TYPE3', u'TYPE7'])
        self.expose_attribute(local_name="description", remote_name="description", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="aggregate_area_range", remote_name="aggregateAreaRange", attribute_type=list, is_required=False, is_unique=False)
        self.expose_attribute(local_name="aggregate_area_range_nssa", remote_name="aggregateAreaRangeNSSA", attribute_type=list, is_required=False, is_unique=False)
        self.expose_attribute(local_name="entity_scope", remote_name="entityScope", attribute_type=str, is_required=False, is_unique=False, choices=[u'ENTERPRISE', u'GLOBAL'])
        self.expose_attribute(local_name="area_id", remote_name="areaID", attribute_type=int, is_required=True, is_unique=False)
        self.expose_attribute(local_name="area_type", remote_name="areaType", attribute_type=str, is_required=False, is_unique=False, choices=[u'NORMAL', u'NSSA', u'STUB'])
        self.expose_attribute(local_name="summaries_enabled", remote_name="summariesEnabled", attribute_type=bool, is_required=False, is_unique=False)
        self.expose_attribute(local_name="suppress_area_range", remote_name="suppressAreaRange", attribute_type=list, is_required=False, is_unique=False)
        self.expose_attribute(local_name="suppress_area_range_nssa", remote_name="suppressAreaRangeNSSA", attribute_type=list, is_required=False, is_unique=False)
        self.expose_attribute(local_name="external_id", remote_name="externalID", attribute_type=str, is_required=False, is_unique=True)
        

        # Fetchers
        
        
        self.metadatas = NUMetadatasFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.global_metadatas = NUGlobalMetadatasFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.ospf_interfaces = NUOSPFInterfacesFetcher.fetcher_with_object(parent_object=self, relationship="child")
        

        self._compute_args(**kwargs)

    # Properties
    
    @property
    def last_updated_by(self):
        """ Get last_updated_by value.

            Notes:
                ID of the user who last updated the object.

                
                This attribute is named `lastUpdatedBy` in VSD API.
                
        """
        return self._last_updated_by

    @last_updated_by.setter
    def last_updated_by(self, value):
        """ Set last_updated_by value.

            Notes:
                ID of the user who last updated the object.

                
                This attribute is named `lastUpdatedBy` in VSD API.
                
        """
        self._last_updated_by = value

    
    @property
    def redistribute_external_enabled(self):
        """ Get redistribute_external_enabled value.

            Notes:
                This flag will determine whether external routes will be redistributed into the area or not. This is enabled only for NSSA areas.

                
                This attribute is named `redistributeExternalEnabled` in VSD API.
                
        """
        return self._redistribute_external_enabled

    @redistribute_external_enabled.setter
    def redistribute_external_enabled(self, value):
        """ Set redistribute_external_enabled value.

            Notes:
                This flag will determine whether external routes will be redistributed into the area or not. This is enabled only for NSSA areas.

                
                This attribute is named `redistributeExternalEnabled` in VSD API.
                
        """
        self._redistribute_external_enabled = value

    
    @property
    def default_metric(self):
        """ Get default_metric value.

            Notes:
                Explicit route cost metric for the default route generated. For STUB areas, it defaults to 1. It is null for the other types of areas.

                
                This attribute is named `defaultMetric` in VSD API.
                
        """
        return self._default_metric

    @default_metric.setter
    def default_metric(self, value):
        """ Set default_metric value.

            Notes:
                Explicit route cost metric for the default route generated. For STUB areas, it defaults to 1. It is null for the other types of areas.

                
                This attribute is named `defaultMetric` in VSD API.
                
        """
        self._default_metric = value

    
    @property
    def default_originate_option(self):
        """ Get default_originate_option value.

            Notes:
                Specifies whether an NSSA area generates a default route, and if it does, whether it is advertised as type 3 LSA or type 7 LSA. If the attribute is set to 'NONE', no default is generated.

                
                This attribute is named `defaultOriginateOption` in VSD API.
                
        """
        return self._default_originate_option

    @default_originate_option.setter
    def default_originate_option(self, value):
        """ Set default_originate_option value.

            Notes:
                Specifies whether an NSSA area generates a default route, and if it does, whether it is advertised as type 3 LSA or type 7 LSA. If the attribute is set to 'NONE', no default is generated.

                
                This attribute is named `defaultOriginateOption` in VSD API.
                
        """
        self._default_originate_option = value

    
    @property
    def description(self):
        """ Get description value.

            Notes:
                Description of OSPFArea

                
        """
        return self._description

    @description.setter
    def description(self, value):
        """ Set description value.

            Notes:
                Description of OSPFArea

                
        """
        self._description = value

    
    @property
    def aggregate_area_range(self):
        """ Get aggregate_area_range value.

            Notes:
                Routes (type 3 LSAs) that belong to networks listed here will be aggregated.

                
                This attribute is named `aggregateAreaRange` in VSD API.
                
        """
        return self._aggregate_area_range

    @aggregate_area_range.setter
    def aggregate_area_range(self, value):
        """ Set aggregate_area_range value.

            Notes:
                Routes (type 3 LSAs) that belong to networks listed here will be aggregated.

                
                This attribute is named `aggregateAreaRange` in VSD API.
                
        """
        self._aggregate_area_range = value

    
    @property
    def aggregate_area_range_nssa(self):
        """ Get aggregate_area_range_nssa value.

            Notes:
                Routes (type 7 LSAs) that belong to networks listed here will be aggregated.

                
                This attribute is named `aggregateAreaRangeNSSA` in VSD API.
                
        """
        return self._aggregate_area_range_nssa

    @aggregate_area_range_nssa.setter
    def aggregate_area_range_nssa(self, value):
        """ Set aggregate_area_range_nssa value.

            Notes:
                Routes (type 7 LSAs) that belong to networks listed here will be aggregated.

                
                This attribute is named `aggregateAreaRangeNSSA` in VSD API.
                
        """
        self._aggregate_area_range_nssa = value

    
    @property
    def entity_scope(self):
        """ Get entity_scope value.

            Notes:
                Specify if scope of entity is Data center or Enterprise level

                
                This attribute is named `entityScope` in VSD API.
                
        """
        return self._entity_scope

    @entity_scope.setter
    def entity_scope(self, value):
        """ Set entity_scope value.

            Notes:
                Specify if scope of entity is Data center or Enterprise level

                
                This attribute is named `entityScope` in VSD API.
                
        """
        self._entity_scope = value

    
    @property
    def area_id(self):
        """ Get area_id value.

            Notes:
                OSPF Area ID

                
                This attribute is named `areaID` in VSD API.
                
        """
        return self._area_id

    @area_id.setter
    def area_id(self, value):
        """ Set area_id value.

            Notes:
                OSPF Area ID

                
                This attribute is named `areaID` in VSD API.
                
        """
        self._area_id = value

    
    @property
    def area_type(self):
        """ Get area_type value.

            Notes:
                Set the OSPF area type. The default value is 'NORMAL', which represents either a backbone area or a standard area. If the areaID is 0, this attribute must be set to 'NORMAL'.

                
                This attribute is named `areaType` in VSD API.
                
        """
        return self._area_type

    @area_type.setter
    def area_type(self, value):
        """ Set area_type value.

            Notes:
                Set the OSPF area type. The default value is 'NORMAL', which represents either a backbone area or a standard area. If the areaID is 0, this attribute must be set to 'NORMAL'.

                
                This attribute is named `areaType` in VSD API.
                
        """
        self._area_type = value

    
    @property
    def summaries_enabled(self):
        """ Get summaries_enabled value.

            Notes:
                This flag determines whether Summaries (Type 3 LSAs) will be redistributed into the area or not. Applicable only to NSSA and Stub area types. Disabling this will make the area a Totally Stub or Totally NSSA area.

                
                This attribute is named `summariesEnabled` in VSD API.
                
        """
        return self._summaries_enabled

    @summaries_enabled.setter
    def summaries_enabled(self, value):
        """ Set summaries_enabled value.

            Notes:
                This flag determines whether Summaries (Type 3 LSAs) will be redistributed into the area or not. Applicable only to NSSA and Stub area types. Disabling this will make the area a Totally Stub or Totally NSSA area.

                
                This attribute is named `summariesEnabled` in VSD API.
                
        """
        self._summaries_enabled = value

    
    @property
    def suppress_area_range(self):
        """ Get suppress_area_range value.

            Notes:
                Routes (type 3 LSAs) that belong to networks listed here will be suppressed.

                
                This attribute is named `suppressAreaRange` in VSD API.
                
        """
        return self._suppress_area_range

    @suppress_area_range.setter
    def suppress_area_range(self, value):
        """ Set suppress_area_range value.

            Notes:
                Routes (type 3 LSAs) that belong to networks listed here will be suppressed.

                
                This attribute is named `suppressAreaRange` in VSD API.
                
        """
        self._suppress_area_range = value

    
    @property
    def suppress_area_range_nssa(self):
        """ Get suppress_area_range_nssa value.

            Notes:
                Routes (type 7 LSAs) that belong to networks listed here will be suppressed.

                
                This attribute is named `suppressAreaRangeNSSA` in VSD API.
                
        """
        return self._suppress_area_range_nssa

    @suppress_area_range_nssa.setter
    def suppress_area_range_nssa(self, value):
        """ Set suppress_area_range_nssa value.

            Notes:
                Routes (type 7 LSAs) that belong to networks listed here will be suppressed.

                
                This attribute is named `suppressAreaRangeNSSA` in VSD API.
                
        """
        self._suppress_area_range_nssa = value

    
    @property
    def external_id(self):
        """ Get external_id value.

            Notes:
                External object ID. Used for integration with third party systems

                
                This attribute is named `externalID` in VSD API.
                
        """
        return self._external_id

    @external_id.setter
    def external_id(self, value):
        """ Set external_id value.

            Notes:
                External object ID. Used for integration with third party systems

                
                This attribute is named `externalID` in VSD API.
                
        """
        self._external_id = value

    

    


# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc, 2017 Nokia
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import logging
logger = logging.getLogger("vspk")

def set_log_level(level, handler=None):
    """ Set both vspk and Bambou log level to the given level

        Args:
            level (logging.level): a logging level
            handler (logging.Handler): a logging handler

        Notes:
            if no handler is provided, it will automatically create a new StreamHandler.

        Examples:
            >>> set_log_level(logging.INFO)
            >>> set_log_level(logging.DEBUG, logging.Streamhandler())
            >>> set_log_level(logging.ERROR)

    """

    from bambou import bambou_logger, pushcenter_logger

    if handler is None:
        handler = logging.StreamHandler()

    bambou_logger.setLevel(level)
    bambou_logger.addHandler(handler)

    pushcenter_logger.setLevel(level)
    pushcenter_logger.addHandler(handler)

    logger.setLevel(level)
    logger.addHandler(handler)


# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc, 2017 Nokia
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.




from .fetchers import NUPermissionsFetcher


from .fetchers import NUMetadatasFetcher


from .fetchers import NUGlobalMetadatasFetcher


from .fetchers import NUEventLogsFetcher

from bambou import NURESTObject


class NULicense(NURESTObject):
    """ Represents a License in the VSD

        Notes:
            Enables retrieval/modification and creation of license files. Most of the attributes are retrieved from the encrypted license. The create API simply provides the encrypted license that is in base64 format.
    """

    __rest_name__ = "license"
    __resource_name__ = "licenses"

    
    ## Constants
    
    CONST_LICENSE_ENCRYPTION_ENCRYPTION_ENABLED = "ENCRYPTION_ENABLED"
    
    CONST_ENTITY_SCOPE_GLOBAL = "GLOBAL"
    
    CONST_ENTITY_SCOPE_ENTERPRISE = "ENTERPRISE"
    
    CONST_LICENSE_TYPE_CLUSTERED = "CLUSTERED"
    
    CONST_LICENSE_TYPE_STANDARD = "STANDARD"
    
    CONST_LICENSE_ENCRYPTION_ENCRYPTION_DISABLED = "ENCRYPTION_DISABLED"
    
    

    def __init__(self, **kwargs):
        """ Initializes a License instance

            Notes:
                You can specify all parameters while calling this methods.
                A special argument named `data` will enable you to load the
                object from a Python dictionary

            Examples:
                >>> license = NULicense(id=u'xxxx-xxx-xxx-xxx', name=u'License')
                >>> license = NULicense(data=my_dict)
        """

        super(NULicense, self).__init__()

        # Read/Write Attributes
        
        self._major_release = None
        self._last_updated_by = None
        self._last_updated_date = None
        self._additional_supported_versions = None
        self._request_id = None
        self._phone = None
        self._license = None
        self._license_encryption = None
        self._license_entities = None
        self._license_id = None
        self._license_type = None
        self._licensed_feature = None
        self._minor_release = None
        self._zip = None
        self._city = None
        self._allowed_avrsgs_count = None
        self._allowed_avrss_count = None
        self._allowed_cpes_count = None
        self._allowed_nics_count = None
        self._allowed_vdfgs_count = None
        self._allowed_vdfs_count = None
        self._allowed_vms_count = None
        self._allowed_vrsgs_count = None
        self._allowed_vrss_count = None
        self._email = None
        self._embedded_metadata = None
        self._encryption_mode = None
        self._unique_license_identifier = None
        self._entity_scope = None
        self._company = None
        self._country = None
        self._creation_date = None
        self._product_version = None
        self._provider = None
        self._is_cluster_license = None
        self._user_name = None
        self._state = None
        self._street = None
        self._customer_key = None
        self._owner = None
        self._expiration_date = None
        self._expiry_timestamp = None
        self._external_id = None
        self._system = None
        
        self.expose_attribute(local_name="major_release", remote_name="majorRelease", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="last_updated_by", remote_name="lastUpdatedBy", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="last_updated_date", remote_name="lastUpdatedDate", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="additional_supported_versions", remote_name="additionalSupportedVersions", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="request_id", remote_name="requestID", attribute_type=str, is_required=False, is_unique=True)
        self.expose_attribute(local_name="phone", remote_name="phone", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="license", remote_name="license", attribute_type=str, is_required=True, is_unique=False)
        self.expose_attribute(local_name="license_encryption", remote_name="licenseEncryption", attribute_type=str, is_required=False, is_unique=False, choices=[u'ENCRYPTION_DISABLED', u'ENCRYPTION_ENABLED'])
        self.expose_attribute(local_name="license_entities", remote_name="licenseEntities", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="license_id", remote_name="licenseID", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="license_type", remote_name="licenseType", attribute_type=str, is_required=False, is_unique=False, choices=[u'CLUSTERED', u'STANDARD'])
        self.expose_attribute(local_name="licensed_feature", remote_name="licensedFeature", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="minor_release", remote_name="minorRelease", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="zip", remote_name="zip", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="city", remote_name="city", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="allowed_avrsgs_count", remote_name="allowedAVRSGsCount", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="allowed_avrss_count", remote_name="allowedAVRSsCount", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="allowed_cpes_count", remote_name="allowedCPEsCount", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="allowed_nics_count", remote_name="allowedNICsCount", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="allowed_vdfgs_count", remote_name="allowedVDFGsCount", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="allowed_vdfs_count", remote_name="allowedVDFsCount", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="allowed_vms_count", remote_name="allowedVMsCount", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="allowed_vrsgs_count", remote_name="allowedVRSGsCount", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="allowed_vrss_count", remote_name="allowedVRSsCount", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="email", remote_name="email", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="embedded_metadata", remote_name="embeddedMetadata", attribute_type=list, is_required=False, is_unique=False)
        self.expose_attribute(local_name="encryption_mode", remote_name="encryptionMode", attribute_type=bool, is_required=False, is_unique=False)
        self.expose_attribute(local_name="unique_license_identifier", remote_name="uniqueLicenseIdentifier", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="entity_scope", remote_name="entityScope", attribute_type=str, is_required=False, is_unique=False, choices=[u'ENTERPRISE', u'GLOBAL'])
        self.expose_attribute(local_name="company", remote_name="company", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="country", remote_name="country", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="creation_date", remote_name="creationDate", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="product_version", remote_name="productVersion", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="provider", remote_name="provider", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="is_cluster_license", remote_name="isClusterLicense", attribute_type=bool, is_required=False, is_unique=False)
        self.expose_attribute(local_name="user_name", remote_name="userName", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="state", remote_name="state", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="street", remote_name="street", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="customer_key", remote_name="customerKey", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="owner", remote_name="owner", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="expiration_date", remote_name="expirationDate", attribute_type=float, is_required=False, is_unique=False)
        self.expose_attribute(local_name="expiry_timestamp", remote_name="expiryTimestamp", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="external_id", remote_name="externalID", attribute_type=str, is_required=False, is_unique=True)
        self.expose_attribute(local_name="system", remote_name="system", attribute_type=str, is_required=False, is_unique=False)
        

        # Fetchers
        
        
        self.permissions = NUPermissionsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.metadatas = NUMetadatasFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.global_metadatas = NUGlobalMetadatasFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.event_logs = NUEventLogsFetcher.fetcher_with_object(parent_object=self, relationship="child")
        

        self._compute_args(**kwargs)

    # Properties
    
    @property
    def major_release(self):
        """ Get major_release value.

            Notes:
                Major software release associated with this license

                
                This attribute is named `majorRelease` in VSD API.
                
        """
        return self._major_release

    @major_release.setter
    def major_release(self, value):
        """ Set major_release value.

            Notes:
                Major software release associated with this license

                
                This attribute is named `majorRelease` in VSD API.
                
        """
        self._major_release = value

    
    @property
    def last_updated_by(self):
        """ Get last_updated_by value.

            Notes:
                ID of the user who last updated the object.

                
                This attribute is named `lastUpdatedBy` in VSD API.
                
        """
        return self._last_updated_by

    @last_updated_by.setter
    def last_updated_by(self, value):
        """ Set last_updated_by value.

            Notes:
                ID of the user who last updated the object.

                
                This attribute is named `lastUpdatedBy` in VSD API.
                
        """
        self._last_updated_by = value

    
    @property
    def last_updated_date(self):
        """ Get last_updated_date value.

            Notes:
                Time stamp when this object was last updated.

                
                This attribute is named `lastUpdatedDate` in VSD API.
                
        """
        return self._last_updated_date

    @last_updated_date.setter
    def last_updated_date(self, value):
        """ Set last_updated_date value.

            Notes:
                Time stamp when this object was last updated.

                
                This attribute is named `lastUpdatedDate` in VSD API.
                
        """
        self._last_updated_date = value

    
    @property
    def additional_supported_versions(self):
        """ Get additional_supported_versions value.

            Notes:
                Indicates additional versions supported by the license.

                
                This attribute is named `additionalSupportedVersions` in VSD API.
                
        """
        return self._additional_supported_versions

    @additional_supported_versions.setter
    def additional_supported_versions(self, value):
        """ Set additional_supported_versions value.

            Notes:
                Indicates additional versions supported by the license.

                
                This attribute is named `additionalSupportedVersions` in VSD API.
                
        """
        self._additional_supported_versions = value

    
    @property
    def request_id(self):
        """ Get request_id value.

            Notes:
                Unique number generated by the License Management system used to identify each license.

                
                This attribute is named `requestID` in VSD API.
                
        """
        return self._request_id

    @request_id.setter
    def request_id(self, value):
        """ Set request_id value.

            Notes:
                Unique number generated by the License Management system used to identify each license.

                
                This attribute is named `requestID` in VSD API.
                
        """
        self._request_id = value

    
    @property
    def phone(self):
        """ Get phone value.

            Notes:
                Phone number of the owner associated with the license file

                
        """
        return self._phone

    @phone.setter
    def phone(self, value):
        """ Set phone value.

            Notes:
                Phone number of the owner associated with the license file

                
        """
        self._phone = value

    
    @property
    def license(self):
        """ Get license value.

            Notes:
                Base 64 value of the license

                
        """
        return self._license

    @license.setter
    def license(self, value):
        """ Set license value.

            Notes:
                Base 64 value of the license

                
        """
        self._license = value

    
    @property
    def license_encryption(self):
        """ Get license_encryption value.

            Notes:
                License encryption

                
                This attribute is named `licenseEncryption` in VSD API.
                
        """
        return self._license_encryption

    @license_encryption.setter
    def license_encryption(self, value):
        """ Set license_encryption value.

            Notes:
                License encryption

                
                This attribute is named `licenseEncryption` in VSD API.
                
        """
        self._license_encryption = value

    
    @property
    def license_entities(self):
        """ Get license_entities value.

            Notes:
                Indicates non enforceable entities associated with the license. 

                
                This attribute is named `licenseEntities` in VSD API.
                
        """
        return self._license_entities

    @license_entities.setter
    def license_entities(self, value):
        """ Set license_entities value.

            Notes:
                Indicates non enforceable entities associated with the license. 

                
                This attribute is named `licenseEntities` in VSD API.
                
        """
        self._license_entities = value

    
    @property
    def license_id(self):
        """ Get license_id value.

            Notes:
                Unique identifier of the license file

                
                This attribute is named `licenseID` in VSD API.
                
        """
        return self._license_id

    @license_id.setter
    def license_id(self, value):
        """ Set license_id value.

            Notes:
                Unique identifier of the license file

                
                This attribute is named `licenseID` in VSD API.
                
        """
        self._license_id = value

    
    @property
    def license_type(self):
        """ Get license_type value.

            Notes:
                None

                
                This attribute is named `licenseType` in VSD API.
                
        """
        return self._license_type

    @license_type.setter
    def license_type(self, value):
        """ Set license_type value.

            Notes:
                None

                
                This attribute is named `licenseType` in VSD API.
                
        """
        self._license_type = value

    
    @property
    def licensed_feature(self):
        """ Get licensed_feature value.

            Notes:
                Indicates the feature supported by the license. Possible value is "vss".

                
                This attribute is named `licensedFeature` in VSD API.
                
        """
        return self._licensed_feature

    @licensed_feature.setter
    def licensed_feature(self, value):
        """ Set licensed_feature value.

            Notes:
                Indicates the feature supported by the license. Possible value is "vss".

                
                This attribute is named `licensedFeature` in VSD API.
                
        """
        self._licensed_feature = value

    
    @property
    def minor_release(self):
        """ Get minor_release value.

            Notes:
                Minor software release for which this license has been issued

                
                This attribute is named `minorRelease` in VSD API.
                
        """
        return self._minor_release

    @minor_release.setter
    def minor_release(self, value):
        """ Set minor_release value.

            Notes:
                Minor software release for which this license has been issued

                
                This attribute is named `minorRelease` in VSD API.
                
        """
        self._minor_release = value

    
    @property
    def zip(self):
        """ Get zip value.

            Notes:
                Zipcode of the owner associated with the license file

                
        """
        return self._zip

    @zip.setter
    def zip(self, value):
        """ Set zip value.

            Notes:
                Zipcode of the owner associated with the license file

                
        """
        self._zip = value

    
    @property
    def city(self):
        """ Get city value.

            Notes:
                City of the owner associated with the license file

                
        """
        return self._city

    @city.setter
    def city(self, value):
        """ Set city value.

            Notes:
                City of the owner associated with the license file

                
        """
        self._city = value

    
    @property
    def allowed_avrsgs_count(self):
        """ Get allowed_avrsgs_count value.

            Notes:
                Maximum number of AVRSGs enabled with this license. A value of -1 indicates an unlimited number of AVRSGs

                
                This attribute is named `allowedAVRSGsCount` in VSD API.
                
        """
        return self._allowed_avrsgs_count

    @allowed_avrsgs_count.setter
    def allowed_avrsgs_count(self, value):
        """ Set allowed_avrsgs_count value.

            Notes:
                Maximum number of AVRSGs enabled with this license. A value of -1 indicates an unlimited number of AVRSGs

                
                This attribute is named `allowedAVRSGsCount` in VSD API.
                
        """
        self._allowed_avrsgs_count = value

    
    @property
    def allowed_avrss_count(self):
        """ Get allowed_avrss_count value.

            Notes:
                Maximum number of AVRSs enabled with this license. A value of -1 indicates an unlimited number of AVRSs

                
                This attribute is named `allowedAVRSsCount` in VSD API.
                
        """
        return self._allowed_avrss_count

    @allowed_avrss_count.setter
    def allowed_avrss_count(self, value):
        """ Set allowed_avrss_count value.

            Notes:
                Maximum number of AVRSs enabled with this license. A value of -1 indicates an unlimited number of AVRSs

                
                This attribute is named `allowedAVRSsCount` in VSD API.
                
        """
        self._allowed_avrss_count = value

    
    @property
    def allowed_cpes_count(self):
        """ Get allowed_cpes_count value.

            Notes:
                Maximum number of NSGs enabled with this license. A value of -1 indicates an unlimited number of NSGs

                
                This attribute is named `allowedCPEsCount` in VSD API.
                
        """
        return self._allowed_cpes_count

    @allowed_cpes_count.setter
    def allowed_cpes_count(self, value):
        """ Set allowed_cpes_count value.

            Notes:
                Maximum number of NSGs enabled with this license. A value of -1 indicates an unlimited number of NSGs

                
                This attribute is named `allowedCPEsCount` in VSD API.
                
        """
        self._allowed_cpes_count = value

    
    @property
    def allowed_nics_count(self):
        """ Get allowed_nics_count value.

            Notes:
                Maximum number of NICs allowed. A value of -1 indicates unlimited number of NICs

                
                This attribute is named `allowedNICsCount` in VSD API.
                
        """
        return self._allowed_nics_count

    @allowed_nics_count.setter
    def allowed_nics_count(self, value):
        """ Set allowed_nics_count value.

            Notes:
                Maximum number of NICs allowed. A value of -1 indicates unlimited number of NICs

                
                This attribute is named `allowedNICsCount` in VSD API.
                
        """
        self._allowed_nics_count = value

    
    @property
    def allowed_vdfgs_count(self):
        """ Get allowed_vdfgs_count value.

            Notes:
                Maximum number of VDFGs enabled with this license. A value of -1 indicates an unlimited number of VDFGs.

                
                This attribute is named `allowedVDFGsCount` in VSD API.
                
        """
        return self._allowed_vdfgs_count

    @allowed_vdfgs_count.setter
    def allowed_vdfgs_count(self, value):
        """ Set allowed_vdfgs_count value.

            Notes:
                Maximum number of VDFGs enabled with this license. A value of -1 indicates an unlimited number of VDFGs.

                
                This attribute is named `allowedVDFGsCount` in VSD API.
                
        """
        self._allowed_vdfgs_count = value

    
    @property
    def allowed_vdfs_count(self):
        """ Get allowed_vdfs_count value.

            Notes:
                Maximum number of VDFs enabled with this license. A value of -1 indicates an unlimited number of VDFs

                
                This attribute is named `allowedVDFsCount` in VSD API.
                
        """
        return self._allowed_vdfs_count

    @allowed_vdfs_count.setter
    def allowed_vdfs_count(self, value):
        """ Set allowed_vdfs_count value.

            Notes:
                Maximum number of VDFs enabled with this license. A value of -1 indicates an unlimited number of VDFs

                
                This attribute is named `allowedVDFsCount` in VSD API.
                
        """
        self._allowed_vdfs_count = value

    
    @property
    def allowed_vms_count(self):
        """ Get allowed_vms_count value.

            Notes:
                Maximum number of VMs enabled with this license. A value of -1 indicates an unlimited number of VMs

                
                This attribute is named `allowedVMsCount` in VSD API.
                
        """
        return self._allowed_vms_count

    @allowed_vms_count.setter
    def allowed_vms_count(self, value):
        """ Set allowed_vms_count value.

            Notes:
                Maximum number of VMs enabled with this license. A value of -1 indicates an unlimited number of VMs

                
                This attribute is named `allowedVMsCount` in VSD API.
                
        """
        self._allowed_vms_count = value

    
    @property
    def allowed_vrsgs_count(self):
        """ Get allowed_vrsgs_count value.

            Notes:
                Maximum number of VRSGs enabled with this license. A value of -1 indicates an unlimited number of VRSGs

                
                This attribute is named `allowedVRSGsCount` in VSD API.
                
        """
        return self._allowed_vrsgs_count

    @allowed_vrsgs_count.setter
    def allowed_vrsgs_count(self, value):
        """ Set allowed_vrsgs_count value.

            Notes:
                Maximum number of VRSGs enabled with this license. A value of -1 indicates an unlimited number of VRSGs

                
                This attribute is named `allowedVRSGsCount` in VSD API.
                
        """
        self._allowed_vrsgs_count = value

    
    @property
    def allowed_vrss_count(self):
        """ Get allowed_vrss_count value.

            Notes:
                Maximum number of VRSs enabled with this license. A value of -1 indicates an unlimited number of VRSs

                
                This attribute is named `allowedVRSsCount` in VSD API.
                
        """
        return self._allowed_vrss_count

    @allowed_vrss_count.setter
    def allowed_vrss_count(self, value):
        """ Set allowed_vrss_count value.

            Notes:
                Maximum number of VRSs enabled with this license. A value of -1 indicates an unlimited number of VRSs

                
                This attribute is named `allowedVRSsCount` in VSD API.
                
        """
        self._allowed_vrss_count = value

    
    @property
    def email(self):
        """ Get email value.

            Notes:
                Email of the owner associated with the license file

                
        """
        return self._email

    @email.setter
    def email(self, value):
        """ Set email value.

            Notes:
                Email of the owner associated with the license file

                
        """
        self._email = value

    
    @property
    def embedded_metadata(self):
        """ Get embedded_metadata value.

            Notes:
                Metadata objects associated with this entity. This will contain a list of Metadata objects if the API request is made using the special flag to enable the embedded Metadata feature. Only a maximum of Metadata objects is returned based on the value set in the system configuration.

                
                This attribute is named `embeddedMetadata` in VSD API.
                
        """
        return self._embedded_metadata

    @embedded_metadata.setter
    def embedded_metadata(self, value):
        """ Set embedded_metadata value.

            Notes:
                Metadata objects associated with this entity. This will contain a list of Metadata objects if the API request is made using the special flag to enable the embedded Metadata feature. Only a maximum of Metadata objects is returned based on the value set in the system configuration.

                
                This attribute is named `embeddedMetadata` in VSD API.
                
        """
        self._embedded_metadata = value

    
    @property
    def encryption_mode(self):
        """ Get encryption_mode value.

            Notes:
                Indicates if the system is associated with a license that allows encryption or not

                
                This attribute is named `encryptionMode` in VSD API.
                
        """
        return self._encryption_mode

    @encryption_mode.setter
    def encryption_mode(self, value):
        """ Set encryption_mode value.

            Notes:
                Indicates if the system is associated with a license that allows encryption or not

                
                This attribute is named `encryptionMode` in VSD API.
                
        """
        self._encryption_mode = value

    
    @property
    def unique_license_identifier(self):
        """ Get unique_license_identifier value.

            Notes:
                Indicates combined string of first 16 and last 16 characters of the license string to be shown in the API

                
                This attribute is named `uniqueLicenseIdentifier` in VSD API.
                
        """
        return self._unique_license_identifier

    @unique_license_identifier.setter
    def unique_license_identifier(self, value):
        """ Set unique_license_identifier value.

            Notes:
                Indicates combined string of first 16 and last 16 characters of the license string to be shown in the API

                
                This attribute is named `uniqueLicenseIdentifier` in VSD API.
                
        """
        self._unique_license_identifier = value

    
    @property
    def entity_scope(self):
        """ Get entity_scope value.

            Notes:
                Specify if scope of entity is Data center or Enterprise level

                
                This attribute is named `entityScope` in VSD API.
                
        """
        return self._entity_scope

    @entity_scope.setter
    def entity_scope(self, value):
        """ Set entity_scope value.

            Notes:
                Specify if scope of entity is Data center or Enterprise level

                
                This attribute is named `entityScope` in VSD API.
                
        """
        self._entity_scope = value

    
    @property
    def company(self):
        """ Get company value.

            Notes:
                Company of the owner associated with the license file

                
        """
        return self._company

    @company.setter
    def company(self, value):
        """ Set company value.

            Notes:
                Company of the owner associated with the license file

                
        """
        self._company = value

    
    @property
    def country(self):
        """ Get country value.

            Notes:
                Country of the owner associated with the license file

                
        """
        return self._country

    @country.setter
    def country(self, value):
        """ Set country value.

            Notes:
                Country of the owner associated with the license file

                
        """
        self._country = value

    
    @property
    def creation_date(self):
        """ Get creation_date value.

            Notes:
                Time stamp when this object was created.

                
                This attribute is named `creationDate` in VSD API.
                
        """
        return self._creation_date

    @creation_date.setter
    def creation_date(self, value):
        """ Set creation_date value.

            Notes:
                Time stamp when this object was created.

                
                This attribute is named `creationDate` in VSD API.
                
        """
        self._creation_date = value

    
    @property
    def product_version(self):
        """ Get product_version value.

            Notes:
                Version of the product that this license applies to

                
                This attribute is named `productVersion` in VSD API.
                
        """
        return self._product_version

    @product_version.setter
    def product_version(self, value):
        """ Set product_version value.

            Notes:
                Version of the product that this license applies to

                
                This attribute is named `productVersion` in VSD API.
                
        """
        self._product_version = value

    
    @property
    def provider(self):
        """ Get provider value.

            Notes:
                Provider of the license file

                
        """
        return self._provider

    @provider.setter
    def provider(self, value):
        """ Set provider value.

            Notes:
                Provider of the license file

                
        """
        self._provider = value

    
    @property
    def is_cluster_license(self):
        """ Get is_cluster_license value.

            Notes:
                Indicates if the license is associated with standlone or cluster setup of VSD

                
                This attribute is named `isClusterLicense` in VSD API.
                
        """
        return self._is_cluster_license

    @is_cluster_license.setter
    def is_cluster_license(self, value):
        """ Set is_cluster_license value.

            Notes:
                Indicates if the license is associated with standlone or cluster setup of VSD

                
                This attribute is named `isClusterLicense` in VSD API.
                
        """
        self._is_cluster_license = value

    
    @property
    def user_name(self):
        """ Get user_name value.

            Notes:
                The name of the user associated with the license

                
                This attribute is named `userName` in VSD API.
                
        """
        return self._user_name

    @user_name.setter
    def user_name(self, value):
        """ Set user_name value.

            Notes:
                The name of the user associated with the license

                
                This attribute is named `userName` in VSD API.
                
        """
        self._user_name = value

    
    @property
    def state(self):
        """ Get state value.

            Notes:
                State of the owner associated with the license file

                
        """
        return self._state

    @state.setter
    def state(self, value):
        """ Set state value.

            Notes:
                State of the owner associated with the license file

                
        """
        self._state = value

    
    @property
    def street(self):
        """ Get street value.

            Notes:
                Address of the owner associated with the license file

                
        """
        return self._street

    @street.setter
    def street(self, value):
        """ Set street value.

            Notes:
                Address of the owner associated with the license file

                
        """
        self._street = value

    
    @property
    def customer_key(self):
        """ Get customer_key value.

            Notes:
                Customer key associated with the licese

                
                This attribute is named `customerKey` in VSD API.
                
        """
        return self._customer_key

    @customer_key.setter
    def customer_key(self, value):
        """ Set customer_key value.

            Notes:
                Customer key associated with the licese

                
                This attribute is named `customerKey` in VSD API.
                
        """
        self._customer_key = value

    
    @property
    def owner(self):
        """ Get owner value.

            Notes:
                Identifies the user that has created this object.

                
        """
        return self._owner

    @owner.setter
    def owner(self, value):
        """ Set owner value.

            Notes:
                Identifies the user that has created this object.

                
        """
        self._owner = value

    
    @property
    def expiration_date(self):
        """ Get expiration_date value.

            Notes:
                Expiration date of this license

                
                This attribute is named `expirationDate` in VSD API.
                
        """
        return self._expiration_date

    @expiration_date.setter
    def expiration_date(self, value):
        """ Set expiration_date value.

            Notes:
                Expiration date of this license

                
                This attribute is named `expirationDate` in VSD API.
                
        """
        self._expiration_date = value

    
    @property
    def expiry_timestamp(self):
        """ Get expiry_timestamp value.

            Notes:
                The Timestamp value of the expiration date of this license

                
                This attribute is named `expiryTimestamp` in VSD API.
                
        """
        return self._expiry_timestamp

    @expiry_timestamp.setter
    def expiry_timestamp(self, value):
        """ Set expiry_timestamp value.

            Notes:
                The Timestamp value of the expiration date of this license

                
                This attribute is named `expiryTimestamp` in VSD API.
                
        """
        self._expiry_timestamp = value

    
    @property
    def external_id(self):
        """ Get external_id value.

            Notes:
                External object ID. Used for integration with third party systems

                
                This attribute is named `externalID` in VSD API.
                
        """
        return self._external_id

    @external_id.setter
    def external_id(self, value):
        """ Set external_id value.

            Notes:
                External object ID. Used for integration with third party systems

                
                This attribute is named `externalID` in VSD API.
                
        """
        self._external_id = value

    
    @property
    def system(self):
        """ Get system value.

            Notes:
                System name information provided by the License Management System.

                
        """
        return self._system

    @system.setter
    def system(self, value):
        """ Set system value.

            Notes:
                System name information provided by the License Management System.

                
        """
        self._system = value

    

    


# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc, 2017 Nokia
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import sys
import json
from collections import OrderedDict
from colorama import init
init()
from colorama import Fore, Style
from tabulate import tabulate
from bambou import NURESTObject


class Printer(object):
    """ Print output for CLI

    """

    TABULATE_FORMAT = "psql"

    @classmethod
    def colorprint(cls, message, color=""):
        """ Print a messsage in a specific color

            Args:
                color: the color of the message
                message: the message to print

        """
        print(color + message + Style.RESET_ALL)

    @classmethod
    def raise_error(cls, message):
        """ Print an error message

            Args:
                message: the message to print

        """
        cls.colorprint("[Error] %s" % message, Fore.RED)
        sys.exit(1)

    @classmethod
    def success(cls, message):
        """ Print a succcess message

            Args:
                message: the message to print

        """
        cls.colorprint("[Success] %s" % message, Fore.GREEN)

    @classmethod
    def warn(cls, message):
        """ Print a warning message

            Args:
                message: the message to print
        """

        cls.colorprint("[WARNING] %s" % message, Fore.YELLOW)

    @classmethod
    def info(cls, message):
        """ Print a log message

            Args:
                message: the message to print
        """

        cls.colorprint("[INFO] %s" % message, Fore.CYAN)

    @classmethod
    def output(cls, data, fields=None, json=False, headers={}):
        """ Print either json or tabulate data

            Args:
                data: the data to display

        """
        if json:
            cls.json(data, fields)
        else:
            cls.tabulate(data, fields, headers)

    ### PRINTING METHODS

    @classmethod
    def json(cls, data, fields):
        """ Print a json version of data

            Args:
                data: something to display

        """
        if isinstance(data, str):
            print(data)

        elif isinstance(data, dict):
            print(json.dumps(data, indent=4))

        elif isinstance(data, list):
            results = []
            for obj in data:
                results.append(cls._object_to_dict(obj, fields))
            print(json.dumps(results, indent=4))

        else:
            print(json.dumps(cls._object_to_dict(data, fields), indent=4))

    @classmethod
    def tabulate(cls, data, fields, headers={}):
        """ Prints a tabulate version of data

            Args:
                data: something to disply

        """
        if isinstance(data, str):
            print tabulate([[data]], tablefmt=Printer.TABULATE_FORMAT)

        elif isinstance(data, dict):
            print tabulate([data], headers=headers, tablefmt=Printer.TABULATE_FORMAT)

        elif isinstance(data, list):
            results = []

            for obj in data:
                if isinstance(obj, NURESTObject):
                    results.append(cls._object_to_dict(obj, fields))
                else:
                    results.append([obj])

            print tabulate(results, headers=headers, tablefmt=Printer.TABULATE_FORMAT)

        else:
            dictionary = cls._object_to_dict(data, fields)
            result = [(key, value) for key, value in dictionary.iteritems()]
            print tabulate(result, headers=headers, tablefmt=Printer.TABULATE_FORMAT)

    @classmethod
    def _object_to_dict(cls, obj, fields=None):
        """ Get object dictionnary with filtered fields

        """
        default_dict = obj.to_dict()

        if fields is None or "ALL" in fields:
            return default_dict

        known_fields = default_dict.keys()
        ordered_dict = OrderedDict()

        for field in fields:
            if field in known_fields:
                ordered_dict[field] = default_dict[field]

        return ordered_dict